@@ -1,9 +1,10 @@
 from __future__ import annotations
+import inspect
 import json
 import sys
 from typed_cap.types import (
     ArgNamed,
-    ArgOpt,
+    ArgOption,
     ArgTypes,
     ArgsParserKeyError,
     ArgsParserMissingArgument,
@@ -11,6 +12,7 @@ from typed_cap.types import (
     ArgsParserOptions,
     ArgsParserUndefinedParser,
     ArgsParserUnexpectedValue,
+    BasicArgOption,
     CapArgKeyNotFound,
     CapInvalidAlias,
     CapInvalidDefaultValue,
@@ -20,25 +22,31 @@ from typed_cap.types import (
     VALID_ALIAS_CANDIDATES,
 )
 from typed_cap.args_parser import args_parser
+from typed_cap.cmt_param import parse_anno_cmt_params
 from typed_cap.typing import (
     VALIDATOR,
     AnnoExtra,
     get_optional_candidates,
     get_queue_type,
     get_type_candidates,
-    typpeddict_parse,
-    typpeddict_parse_extra,
+    argstyping_parse,
+    argstyping_parse_extra,
 )
 from typed_cap.utils import (
+    RO,
     flatten,
     get_terminal_width,
+    is_T_based,
     panic,
     split_by_length,
-    to_red,
-    to_yellow,
-    to_blue,
     unwrap_or,
 )
+from typed_cap.utils.code import (
+    get_all_comments_parameters,
+    get_annotations,
+    get_docs_from_annotations,
+)
+from typed_cap.utils.color import Colors, fg
 from typing import (
     Any,
     Callable,
@@ -54,20 +62,15 @@ from typing import (
     TypeVar,
     TypedDict,
     Union,
+    get_args,
 )
 
 
 ArgCallback = Callable[["Cap", List[List]], Union[NoReturn, List[List]]]
 
 
-class _ArgOpt(TypedDict):
-    val: Optional[Any]
-    type: Type
-    about: Optional[str]
-    alias: Optional[VALID_ALIAS_CANDIDATES]
-    cb: Optional[ArgCallback]
-    cb_idx: int
-    hide: bool
+# _ArgOpt = ArgOption[ArgCallback]
+_ArgOpt = ArgOption
 
 
 class _ParsedVal(TypedDict):
@@ -76,18 +79,41 @@ class _ParsedVal(TypedDict):
     queue_type: Optional[Literal["list", "tuple"]]
 
 
-T = TypeVar("T", bound=TypedDict)
+T = TypeVar("T", bound=Union[TypedDict, object])
+
+
+class _GVCS(Generic[T]):
+    _t_based: Union[Type[dict], Type[object]]
+    _gvc: T
+
+    def __init__(self, t: Union[Type[dict], Type[object]], val_ctr: T) -> None:
+        self._t_based = t
+        self._gvc = val_ctr
+
+    def setVal(self, key: str, val: Any):
+        if self._t_based is dict:
+            self._gvc[key] = val  # type: ignore
+        elif self._t_based is object:
+            self._gvc.__setattr__(key, val)
 
 
 class Parsed(Generic[T]):
+    _argstype: Type[T]
+    _args_obj: Optional[T]
     _args: List[str]
     _parsed_map: Dict[str, _ParsedVal]
 
     def __init__(
-        self, args: List[str], parsed_map: Dict[str, _ParsedVal]
+        self,
+        argstype: Type[T],
+        args: List[str],
+        parsed_map: Dict[str, _ParsedVal],
+        args_obj: Optional[T],
     ) -> None:
+        self._argstype = argstype
         self._args = args
         self._parsed_map = parsed_map
+        self._args_obj = args_obj
 
     @property
     def arguments(self) -> List[str]:
@@ -99,19 +125,32 @@ class Parsed(Generic[T]):
 
     @property
     def value(self) -> T:
-        val = {}
+        val: T
+        gvc: _GVCS
+        t_based = is_T_based(self._argstype)
+        if t_based is dict:
+            val = {}  # type: ignore
+            gvc = _GVCS(dict, val)
+        elif t_based is object:
+            if self._args_obj is None:
+                raise Unhandled("args_obj is None", "Parsed.value")
+            val = self._args_obj
+            gvc = _GVCS(object, val)
+        else:
+            raise Unhandled()
+
         for key, parsed in self._parsed_map.items():
             pv = parsed["val"]
             pv = flatten(pv)
             if len(pv) == 0:
-                val[key] = parsed["default_val"]
+                gvc.setVal(key, parsed["default_val"])
             elif parsed["queue_type"] == "list":
-                val[key] = flatten(pv)
+                gvc.setVal(key, flatten(pv))
             elif parsed["queue_type"] == "tuple":
-                val[key] = pv[-1]
+                gvc.setVal(key, pv[-1])
             else:
-                val[key] = pv[-1]
-        return val  # type: ignore
+                gvc.setVal(key, pv[-1])
+        return val
 
     @property
     def val(self) -> T:
@@ -176,16 +215,36 @@ def _helper_help_cb(c: "Cap", v: List[List[bool]]) -> NoReturn:
             prefix_width + MIN_ABOUT_WIDTH,
         )
         remain_width = width - prefix_width
+
         for key, ln in arg_lns:
-            about = unwrap_or(c._args[key]["about"], "")
+            about = []
+            if c._args[key]["about"] is not None:
+                about.append(c._args[key]["about"])
+
+            default_val = None
+            if c._args[key]["val"] is None:
+                if c._args[key]["cls_attr_val"] is not None:
+                    default_val = str(c._args[key]["cls_attr_val"])
+            else:
+                default_val = str(c._args[key]["val"])
+            if default_val is not None and c._args[key]["show_default"]:
+                about.append(f"(default: {default_val})")
+
             about = split_by_length(
-                about, remain_width, add_hyphen=True, remove_leading_space=True
+                " ".join(about),
+                remain_width,
+                add_hyphen=True,
+                remove_leading_space=True,
             )
+
+            if len(about) == 0:
+                about = [""]
             for i, abt in enumerate(about):
                 if i == 0:
                     lns.append((1, ln.ljust(max_opt_len + 4) + abt))
                 else:
                     lns.append((1, "".ljust(prefix_width) + abt))
+
         for indent, ln in lns:
             print("".ljust(indent * INDENT_SIZE) + ln)
     exit(0)
@@ -254,6 +313,7 @@ class _Helpers(TypedDict):
     arg_version: _Helper_fn
 
 
+# TODO: remove this
 helpers: _Helpers = {
     "arg_help": helper_arg_help,
     "arg_version": helper_arg_version,
@@ -263,19 +323,19 @@ helpers: _Helpers = {
 def colorize_text_t_type(t: Type) -> str:
     tn = ""
     try:
-        tn = t.__name__
+        tn: str = t.__name__
     except AttributeError:
         tn = str(t)
-    return to_blue(tn)
+    return str(fg(tn, Colors.Blue))
 
 
 def colorize_text_t_option_name(key: str) -> str:
-    return to_yellow(key)
+    return str(fg(key, Colors.Yellow))
 
 
 def colorize_text_t_value(val: Any) -> str:
     try:
-        return to_red(str(val))
+        return str(fg(val, Colors.Red))
     except Exception as err:
         print(err)
         return "[...]"
@@ -285,25 +345,67 @@ def colorize_text_t_value(val: Any) -> str:
 class Cap(Generic[K, T, U]):
     _argstype: Type[T]
     _args: Dict[str, _ArgOpt]
-    _delimiter: Optional[str] = ","
     _about: Optional[str]
+    _delimiter: RO[str]
     _name: Optional[str]
     _version: Optional[str]
     _raw_err: bool
     _preset_helper_used: bool
+    # cap options
+    stop_at_type: Optional[type]
+    _add_helper_help: bool
+
+    @staticmethod
+    def helpers() -> _Helpers:
+        return helpers
 
     def __init__(
         self,
         argstype: Type[T],
+        stop_at_type: Optional[type] = None,
+        use_cls_doc_as_about: bool = True,
+        use_anno_doc_as_about: bool = True,
+        use_anno_cmt_params: bool = True,
+        add_helper_help: bool = True,
     ) -> None:
         self._argstype = argstype
         self._args = {}
         self._about = None
+        self._delimiter: RO[str] = RO.Some(",")
         self._name = None
         self._version = None
         self._raw_err = False
         self._preset_helper_used = False
+        #
+        self.stop_at_type = stop_at_type
+        #
         self._parse_argstype()
+        self._parse_anno_details()
+
+        if use_cls_doc_as_about:
+            self._about = inspect.getdoc(self._argstype)
+
+        if use_anno_doc_as_about:
+            for name, opt in self._args.items():
+                self._args[name]["about"] = unwrap_or(opt["about"], opt["doc"])
+
+        if use_anno_cmt_params:
+            named_params = parse_anno_cmt_params(self._args)
+            for name, params in named_params.items():
+                #
+                alias = params.get("alias", None)
+                if alias is not None:
+                    self._set_alias(name, alias)
+                #
+                delimiter = params.get("delimiter", None)
+                if delimiter is not None:
+                    self._args[name]["local_delimiter"] = delimiter
+                #
+                show_default = params.get("show_default", None)
+                if show_default is not None:
+                    self._args[name]["show_default"] = show_default
+
+        self._add_helper_help = add_helper_help
 
     def _get_key(self, name: str) -> Union[NoReturn, str]:
         for key, opt in self._args.items():
@@ -321,6 +423,8 @@ class Cap(Generic[K, T, U]):
             raise CapArgKeyNotFound(key)
         else:
             if alias is not None:
+                # if alias not in get_args(VALID_ALIAS_CANDIDATES):
+                #     raise CapInvalidAlias(key, alias)
                 try:
                     self._get_key(alias)
                     raise CapInvalidAlias(key, alias)
@@ -337,20 +441,38 @@ class Cap(Generic[K, T, U]):
             err_msg = f"{title}: {msg}\n\t{err.__class__.__name__}"
             panic(err_msg)
 
+    def _parse_anno_details(self):
+        annos = get_annotations(self._argstype, stop_at=self.stop_at_type)
+        named_doc = get_docs_from_annotations(annos)
+        named_cmt_params = get_all_comments_parameters(annos)
+        for name, doc in named_doc.items():
+            self._args[name]["doc"] = doc
+        for name, cmt_params in named_cmt_params.items():
+            self._args[name]["cmt_params"] = cmt_params
+
     def _parse_argstype(self):
         typed: Dict[str, Type]
         extra: Optional[Dict[str, AnnoExtra]] = None
         if sys.version_info.minor < 9:
-            typed = typpeddict_parse(self._argstype)
+            typed = argstyping_parse(self._argstype)
         else:
-            typed, extra = typpeddict_parse_extra(self._argstype)
+            typed, extra = argstyping_parse_extra(self._argstype)
 
         for key, t in typed.items():
+            attr_val = None
+            try:
+                attr_val = self._argstype.__getattribute__(self._argstype, key)  # type: ignore
+            except AttributeError:
+                ...
+            except TypeError:
+                # TypeError: descriptor '__getattribute__' requires a 'dict' object but received a '_TypedDictMeta'
+                ...
             self.add_argument(
                 key,
                 arg_type=t,
                 callback_priority=0,
                 hide=False,
+                cls_attr_val=attr_val,
                 prevent_overwrite=False,
                 ignore_invalid_alias=False,
             )
@@ -368,6 +490,9 @@ class Cap(Generic[K, T, U]):
         callback: Optional[ArgCallback] = None,
         callback_priority: int = 1,
         hide: bool = False,
+        doc: Optional[str] = None,
+        show_default: bool = True,
+        cls_attr_val: Optional[Any] = None,
         prevent_overwrite: bool = False,
         ignore_invalid_alias: bool = False,
     ) -> Cap:
@@ -382,6 +507,11 @@ class Cap(Generic[K, T, U]):
             "cb": callback,
             "cb_idx": callback_priority,
             "hide": hide,
+            "doc": doc,
+            "cmt_params": {},
+            "show_default": show_default,
+            "cls_attr_val": cls_attr_val,
+            "local_delimiter": RO.NONE(),
         }
         if alias is not None:
             try:
@@ -393,8 +523,13 @@ class Cap(Generic[K, T, U]):
                     raise err
         return self
 
-    def set_delimiter(self, delimiter: Optional[str]) -> Cap:
-        self._delimiter = delimiter
+    def set_delimiter(self, delimiter: str) -> Cap:
+        # TODO: re support
+        if len(delimiter) == 0:
+            raise ValueError(
+                "Invalid delimiter; length of delimiter:<str> larger than 0"
+            )
+        self._delimiter = RO.Some(delimiter)
         return self
 
     def set_callback(
@@ -433,34 +568,40 @@ class Cap(Generic[K, T, U]):
         return self.default_strict(value)  # type: ignore[arg-type]
 
     def default_strict(self, value: T) -> Cap:
-        for arg, val in value.items():
-            try:
-                t = self._args[arg]["type"]
-                valid, _, _ = VALIDATOR.extract(t, val, cvt=False)
-                if valid:
-                    self._args[arg]["val"] = val
-                else:
-                    # raise CapInvalidDefaultValue(arg, t, val)
+        if is_T_based(self._argstype) is object:
+            # TODO: TBD: should default be available for object-based?
+            print(
+                "[warn] `default` has been ignore since cap using an object-based argstype"
+            )
+        else:
+            for arg, val in value.items():  # type: ignore
+                try:
+                    t = self._args[arg]["type"]
+                    valid, _, _ = VALIDATOR.extract(t, val, cvt=False)
+                    if valid:
+                        self._args[arg]["val"] = val
+                    else:
+                        # raise CapInvalidDefaultValue(arg, t, val)
+                        self._panic(
+                            f"invalid default value {colorize_text_t_value(val)} for option {colorize_text_t_option_name(arg)}:{colorize_text_t_type(t)}",
+                            "Cap.default_strict",
+                            CapInvalidDefaultValue(arg, t, val),
+                        )
+                except KeyError as err:
+                    name = str(err)
                     self._panic(
-                        f"invalid default value {colorize_text_t_value(val)} for option {colorize_text_t_option_name(arg)}:{colorize_text_t_type(t)}",
+                        f"unknown named argument {colorize_text_t_option_name(name)}",
                         "Cap.default_strict",
-                        CapInvalidDefaultValue(arg, t, val),
+                        CapUnknownArg(name, "default_strict"),
+                    )
+                except Exception as err:
+                    raise Unhandled(
+                        desc=f"unknown issue: {err.__class__.__name__}",
+                        loc="Cap.default_strict",
                     )
-            except KeyError as err:
-                name = str(err)
-                self._panic(
-                    f"unknown named argument {colorize_text_t_option_name(name)}",
-                    "Cap.default_strict",
-                    CapUnknownArg(name, "default_strict"),
-                )
-            except Exception as err:
-                raise Unhandled(
-                    desc=f"unknown issue: {err.__class__.__name__}",
-                    loc="Cap.default_strict",
-                )
         return self
 
-    def helper(self, helpers: Dict[K, ArgOpt]) -> Cap:
+    def helper(self, helpers: Dict[K, BasicArgOption]) -> Cap:
         if self._preset_helper_used:
             print(
                 "[warn] detected call of `Cap.helper` after call of preset helpers"
@@ -490,11 +631,20 @@ class Cap(Generic[K, T, U]):
                 )
         return self
 
+    def _before_parse(self):
+        if self._add_helper_help:
+            if self._args.get("help") is None:
+                self.helpers()["arg_help"](self, "help")
+
     def parse(
         self,
         argv: List[str] = sys.argv[1:],
         args_parser_options: Optional[ArgsParserOptions] = None,
     ) -> Parsed[T]:
+        self._before_parse()
+
+        VALIDATOR.delimiter = self._delimiter
+
         def _is_flag(t: Type) -> bool:
             if t == bool:
                 return True
@@ -556,7 +706,14 @@ class Cap(Generic[K, T, U]):
             )
             for v in val:
                 t = opt["type"]
-                valid, v_got, err = VALIDATOR.extract(t, v, cvt=True)
+                temp_delimiter = opt["local_delimiter"]
+                valid, v_got, err = VALIDATOR.extract(
+                    t,
+                    v,
+                    cvt=True,
+                    temp_delimiter=temp_delimiter,
+                    leave_scope=True,
+                )
                 # TODO: catch extract failed
                 if valid:
                     parsed["val"].append([v_got])
@@ -576,10 +733,11 @@ class Cap(Generic[K, T, U]):
         cb_list = sorted(cb_list, key=lambda x: x[1])
         cb_list.reverse()
         for key, _ in cb_list:
-            parsed = parsed_map.get(key)
-            if parsed is None:
+            _p = parsed_map.get(key)
+            if _p is None:
                 continue
             else:
+                parsed = _p
                 if len(parsed["val"]) >= 0:
                     try:
                         arg = self._args[key]
@@ -590,6 +748,10 @@ class Cap(Generic[K, T, U]):
                         continue
 
         # assign default value to empty field
+        args_obj: Optional[T] = None
+        T_based = is_T_based(self._argstype)
+        if T_based is object:
+            args_obj = self._argstype()  # type: ignore
         for key, opt in self._args.items():
             if opt["hide"]:
                 if parsed_map.get(key) is not None:
@@ -606,7 +768,18 @@ class Cap(Generic[K, T, U]):
                         ),
                     }
                     if (
-                        opt["val"] is None
+                        parsed_map[key]["default_val"] is None
+                        and T_based is object
+                        and args_obj is not None
+                    ):
+                        try:
+                            parsed_map[key][
+                                "default_val"
+                            ] = args_obj.__getattribute__(key)
+                        except AttributeError:
+                            pass
+                    if (
+                        parsed_map[key]["default_val"] is None
                         and get_optional_candidates(opt["type"]) is None
                     ):
                         self._panic(
@@ -615,4 +788,4 @@ class Cap(Generic[K, T, U]):
                             ArgsParserMissingArgument(key, opt["type"]),
                         )
 
-        return Parsed(out["args"], parsed_map)
+        return Parsed(self._argstype, out["args"], parsed_map, args_obj)
